@@ -13,39 +13,59 @@ insert_header = function(doc) {
 }
 
 # Makes latex header with macros required for highlighting, tikz and framed
-make_header_latex = function() {
-  h = paste(c(
-    sprintf('\\usepackage[%s]{graphicx}\\usepackage[%s]{color}',
-            opts_knit$get('latex.options.graphicx') %n% '',
-            opts_knit$get('latex.options.color') %n% ''),
+make_header_latex = function(doc) {
+  h = one_string(c(
+    header_latex_packages(doc),
     .header.maxwidth, opts_knit$get('header'),
     if (getOption('OutDec') != '.') '\\usepackage{amsmath}',
     if (out_format('latex')) '\\usepackage{alltt}'
-  ), collapse = '\n')
+  ))
   if (opts_knit$get('self.contained')) h else {
-    writeLines(h, 'knitr.sty')
+    write_utf8(h, 'knitr.sty')
     '\\usepackage{knitr}'
   }
 }
 
+# if the document already contains \usepackage[options]{pkg}, use the same
+# options to avoid the option clash error in LaTeX
+use_package = function(pkg, doc) {
+  opts = sapply(pkg, function(p) {
+    r = sprintf('.*?\\\\usepackage\\[(.+?)]\\{%s}.*', p)
+    o = xfun::grep_sub(r, '\\1', doc)
+    if (length(o)) return(o[1])
+    opts_knit$get(paste0('latex.options.', p)) %n% ''
+  })
+  sprintf('\\usepackage[%s]{%s}', opts, pkg)
+}
+
+# for backward-compatibility, use xcolor package unless the latex.options.color
+# has been set; xcolor is preferred: https://github.com/latex3/latex2e/pull/719
+header_latex_packages = function(doc) {
+  paste(use_package(c('graphicx', 'xcolor'), doc), collapse = '')
+}
+
 insert_header_latex = function(doc, b) {
   i = grep(b, doc)
   if (length(i) >= 1L) {
     # it is safer to add usepackage{upquote} before begin{document} than after
     # documentclass{article} because it must appear after usepackage{fontenc};
     # see this weird problem: http://stackoverflow.com/q/12448507/559676
-    if (!out_format('listings') && length(j <- grep(p <- '(\\s*)(\\\\begin\\{document\\})', doc)[1L])) {
-      doc[j] = sub(p, '\n\\\\IfFileExists{upquote.sty}{\\\\usepackage{upquote}}{}\n\\2', doc[j])
+    p = '(?<!%)(\\s*)(\\\\begin\\{document\\})'
+    if (!out_format('listings') && length(j <- grep(p, doc, perl = TRUE))) {
+      j = j[1]
+      doc[j] = sub(p, '\n\\\\IfFileExists{upquote.sty}{\\\\usepackage{upquote}}{}\n\\2', doc[j], perl = TRUE)
     }
     i = i[1L]; l = stringr::str_locate(doc[i], b)
     tmp = stringr::str_sub(doc[i], l[, 1], l[, 2])
-    stringr::str_sub(doc[i], l[,1], l[,2]) = paste0(tmp, make_header_latex())
+    stringr::str_sub(doc[i], l[,1], l[,2]) = paste0(tmp, make_header_latex(doc))
   } else if (parent_mode() && !child_mode()) {
     # in parent mode, we fill doc to be a complete document
-    doc[1L] = paste(c(getOption('tikzDocumentDeclaration'), make_header_latex(),
-                      .knitEnv$tikzPackages, '\\begin{document}', doc[1L]), collapse = '\n')
-    doc[length(doc)] = paste(
-      c(doc[length(doc)], .knitEnv$bibliography, '\\end{document}'), collapse = '\n'
+    doc[1L] = one_string(c(
+      getOption('tikzDocumentDeclaration'), make_header_latex(doc),
+      .knitEnv$tikzPackages, '\\begin{document}', doc[1L]
+    ))
+    doc[length(doc)] = one_string(
+      c(doc[length(doc)], .knitEnv$bibliography, '\\end{document}')
     )
   }
   doc
@@ -54,11 +74,11 @@ insert_header_latex = function(doc, b) {
 make_header_html = function() {
   h = opts_knit$get('header')
   h = h[setdiff(names(h), c('tikz', 'framed'))]
-  if (opts_knit$get('self.contained')) {
-    paste(c('<style type="text/css">', h[['highlight']], '</style>',
-            unlist(h[setdiff(names(h), 'highlight')])), collapse = '\n')
-  } else {
-    writeLines(h, 'knitr.css')
+  if (opts_knit$get('self.contained')) one_string(c(
+    '<style type="text/css">', h[['highlight']], '</style>',
+    unlist(h[setdiff(names(h), 'highlight')])
+  )) else {
+    write_utf8(h, 'knitr.css')
     '<link rel="stylesheet" href="knitr.css" type="text/css" />'
   }
 }
@@ -111,7 +131,7 @@ set_header = function(...) {
 # header for Latex Syntax Highlighting
 .header.hi.tex = theme_to_header_latex(.default.sty)$highlight
 .knitr.sty = inst_dir('misc', 'knitr.sty')
-.header.framed = paste(readLines(.knitr.sty), collapse = '\n')
+.header.framed = one_string(read_utf8(.knitr.sty))
 # CSS for html syntax highlighting
 .header.hi.html = theme_to_header_html(.default.sty)$highlight
 rm(list = c('.default.sty', '.knitr.sty')) # do not need them any more
@@ -123,8 +143,8 @@ rm(list = c('.default.sty', '.knitr.sty')) # do not need them any more
 '
 
 .header.maxwidth =
-'%% maxwidth is the original width if it is less than linewidth
-%% otherwise use linewidth (to make sure the graphics do not exceed the margin)
+'% maxwidth is the original width if it is less than linewidth
+% otherwise use linewidth (to make sure the graphics do not exceed the margin)
 \\makeatletter
 \\def\\maxwidth{ %
   \\ifdim\\Gin@nat@width>\\linewidth
