@@ -442,15 +442,6 @@ sew = function(x, options = list(), ...) {
   UseMethod('sew', x)
 }
 
-# TODO: see if we can remove this function in the future
-wrap = function(...) {
-  warning2(
-    'The internal function knitr:::wrap() has been deprecated. Please use the ',
-    'exported function knitr::sew() instead.'
-  )
-  sew(...)
-}
-
 #' @export
 sew.list = function(x, options = list(), ...) {
   if (length(x) == 0L) return(x)
@@ -593,15 +584,6 @@ sew.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE, .
   })), collapse = '')
 }
 
-# TODO: remove this in the future
-wrap.knit_image_paths = function(...) {
-  warning2(
-    "The internal function knitr:::wrap.knit_image_paths has been deprecated. ",
-    "Please use knitr::sew() on an object of the class 'knit_image_paths' instead."
-  )
-  sew.knit_image_paths(...)
-}
-
 #' @export
 sew.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE, ...) {
   ext = x$extension
