@@ -31,6 +31,8 @@
 #'
 #' # or convert markdown to HTML
 #' stitch(s, system.file('misc', 'knitr-template.Rmd', package = 'knitr'))
+#'
+#' unlink(c('stitch-test.html', 'stitch-test.md', 'figure'), recursive = TRUE)
 stitch = function(script,
                   template = system.file('misc', 'knitr-template.Rnw', package = 'knitr'),
                   output = NULL, text = NULL, envir = parent.frame()) {
@@ -69,7 +71,7 @@ stitch = function(script,
     message('PDF output at: ', with_ext(out, 'pdf'))
   }, md = {
     out.html = with_ext(out, 'html')
-    markdown::markdownToHTML(out, out.html)
+    markdown::mark_html(out, out.html)
     message('HTML output at: ', out.html)
   })
   out
